@@ -3,18 +3,17 @@
  * initsplan.c
  *	  Target list, qualification, joininfo initialization routines
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/plan/initsplan.c,v 1.65 2001/10/25 05:49:33 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/optimizer/plan/initsplan.c,v 1.75 2002/09/04 20:31:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
-#include <sys/types.h>
 
 #include "catalog/pg_operator.h"
 #include "catalog/pg_type.h"
@@ -30,7 +29,7 @@
 #include "parser/parsetree.h"
 #include "parser/parse_expr.h"
 #include "parser/parse_oper.h"
-#include "parser/parse_type.h"
+#include "utils/builtins.h"
 #include "utils/lsyscache.h"
 #include "utils/syscache.h"
 
@@ -51,6 +50,86 @@ static void check_mergejoinable(RestrictInfo *restrictinfo);
 static void check_hashjoinable(RestrictInfo *restrictinfo);
 
 
+/*****************************************************************************
+ *
+ *	 JOIN TREES
+ *
+ *****************************************************************************/
+
+/*
+ * add_base_rels_to_query
+ *
+ *	  Scan the query's jointree and create baserel RelOptInfos for all
+ *	  the base relations (ie, table and subquery RTEs) appearing in the
+ *	  jointree.  Also, create otherrel RelOptInfos for join RTEs.
+ *
+ * The return value is a list of all the baserel indexes (but not join RTE
+ * indexes) included in the scanned jointree.  This is actually just an
+ * internal convenience for marking join otherrels properly; no outside
+ * caller uses the result.
+ *
+ * At the end of this process, there should be one baserel RelOptInfo for
+ * every non-join RTE that is used in the query.  Therefore, this routine
+ * is the only place that should call build_base_rel.  But build_other_rel
+ * will be used again later to build rels for inheritance children.
+ */
+List *
+add_base_rels_to_query(Query *root, Node *jtnode)
+{
+	List	   *result = NIL;
+
+	if (jtnode == NULL)
+		return NIL;
+	if (IsA(jtnode, RangeTblRef))
+	{
+		int			varno = ((RangeTblRef *) jtnode)->rtindex;
+
+		build_base_rel(root, varno);
+		result = makeListi1(varno);
+	}
+	else if (IsA(jtnode, FromExpr))
+	{
+		FromExpr   *f = (FromExpr *) jtnode;
+		List	   *l;
+
+		foreach(l, f->fromlist)
+		{
+			result = nconc(result,
+						   add_base_rels_to_query(root, lfirst(l)));
+		}
+	}
+	else if (IsA(jtnode, JoinExpr))
+	{
+		JoinExpr   *j = (JoinExpr *) jtnode;
+		RelOptInfo *jrel;
+
+		result = add_base_rels_to_query(root, j->larg);
+		result = nconc(result,
+					   add_base_rels_to_query(root, j->rarg));
+		/* the join's own rtindex is NOT added to result */
+		jrel = build_other_rel(root, j->rtindex);
+
+		/*
+		 * Mark the join's otherrel with outerjoinset = list of baserel
+		 * ids included in the join.  Note we must copy here because
+		 * result list is destructively modified by nconcs at higher
+		 * levels.
+		 */
+		jrel->outerjoinset = listCopy(result);
+
+		/*
+		 * Safety check: join RTEs should not be SELECT FOR UPDATE targets
+		 */
+		if (intMember(j->rtindex, root->rowMarks))
+			elog(ERROR, "SELECT FOR UPDATE cannot be applied to a join");
+	}
+	else
+		elog(ERROR, "add_base_rels_to_query: unexpected node type %d",
+			 nodeTag(jtnode));
+	return result;
+}
+
+
 /*****************************************************************************
  *
  *	 TARGET LISTS
@@ -59,10 +138,8 @@ static void check_hashjoinable(RestrictInfo *restrictinfo);
 
 /*
  * build_base_rel_tlists
- *	  Creates rel nodes for every relation mentioned in the target list
- *	  'tlist' (if a node hasn't already been created) and adds them to
- *	  root->base_rel_list.	Creates targetlist entries for each var seen
- *	  in 'tlist' and adds them to the tlist of the appropriate rel node.
+ *	  Creates targetlist entries for each var seen in 'tlist' and adds
+ *	  them to the tlist of the appropriate rel node.
  */
 void
 build_base_rel_tlists(Query *root, List *tlist)
@@ -75,9 +152,13 @@ build_base_rel_tlists(Query *root, List *tlist)
 
 /*
  * add_vars_to_targetlist
- *	  For each variable appearing in the list, add it to the relation's
- *	  targetlist if not already present.  Corresponding base rel nodes
- *	  will be created if not already present.
+ *	  For each variable appearing in the list, add it to the owning
+ *	  relation's targetlist if not already present.
+ *
+ * Note that join alias variables will be attached to the otherrel for
+ * the join RTE.  They will later be transferred to the tlist of
+ * the corresponding joinrel.  We will also cause entries to be made
+ * for the Vars that the alias will eventually depend on.
  */
 static void
 add_vars_to_targetlist(Query *root, List *vars)
@@ -87,70 +168,23 @@ add_vars_to_targetlist(Query *root, List *vars)
 	foreach(temp, vars)
 	{
 		Var		   *var = (Var *) lfirst(temp);
-		RelOptInfo *rel = build_base_rel(root, var->varno);
+		RelOptInfo *rel = find_base_rel(root, var->varno);
 
 		add_var_to_tlist(rel, var);
-	}
-}
 
-/*----------
- * add_missing_rels_to_query
- *
- *	  If we have a relation listed in the join tree that does not appear
- *	  in the target list nor qualifications, we must add it to the base
- *	  relation list so that it can be processed.  For instance,
- *			select count(*) from foo;
- *	  would fail to scan foo if this routine were not called.  More subtly,
- *			select f.x from foo f, foo f2
- *	  is a join of f and f2.  Note that if we have
- *			select foo.x from foo f
- *	  this also gets turned into a join (between foo as foo and foo as f).
- *
- *	  Returns a list of all the base relations (RelOptInfo nodes) that appear
- *	  in the join tree.  This list can be used for cross-checking in the
- *	  reverse direction, ie, that we have a join tree entry for every
- *	  relation used in the query.
- *----------
- */
-List *
-add_missing_rels_to_query(Query *root, Node *jtnode)
-{
-	List	   *result = NIL;
-
-	if (jtnode == NULL)
-		return NIL;
-	if (IsA(jtnode, RangeTblRef))
-	{
-		int			varno = ((RangeTblRef *) jtnode)->rtindex;
-
-		/* This call to build_base_rel does the primary work... */
-		RelOptInfo *rel = build_base_rel(root, varno);
-
-		result = makeList1(rel);
-	}
-	else if (IsA(jtnode, FromExpr))
-	{
-		FromExpr   *f = (FromExpr *) jtnode;
-		List	   *l;
-
-		foreach(l, f->fromlist)
+		if (rel->reloptkind == RELOPT_OTHER_JOIN_REL)
 		{
-			result = nconc(result,
-						   add_missing_rels_to_query(root, lfirst(l)));
+			/* Var is an alias */
+			Node	   *expansion;
+			List	   *varsused;
+
+			expansion = flatten_join_alias_vars((Node *) var,
+												root->rtable, true);
+			varsused = pull_var_clause(expansion, false);
+			add_vars_to_targetlist(root, varsused);
+			freeList(varsused);
 		}
 	}
-	else if (IsA(jtnode, JoinExpr))
-	{
-		JoinExpr   *j = (JoinExpr *) jtnode;
-
-		result = add_missing_rels_to_query(root, j->larg);
-		result = nconc(result,
-					   add_missing_rels_to_query(root, j->rarg));
-	}
-	else
-		elog(ERROR, "add_missing_rels_to_query: unexpected node type %d",
-			 nodeTag(jtnode));
-	return result;
 }
 
 
@@ -165,10 +199,9 @@ add_missing_rels_to_query(Query *root, Node *jtnode)
  * distribute_quals_to_rels
  *	  Recursively scan the query's join tree for WHERE and JOIN/ON qual
  *	  clauses, and add these to the appropriate RestrictInfo and JoinInfo
- *	  lists belonging to base RelOptInfos.	New base rel entries are created
- *	  as needed.  Also, base RelOptInfos are marked with outerjoinset
- *	  information, to aid in proper positioning of qual clauses that appear
- *	  above outer joins.
+ *	  lists belonging to base RelOptInfos.	Also, base RelOptInfos are marked
+ *	  with outerjoinset information, to aid in proper positioning of qual
+ *	  clauses that appear above outer joins.
  *
  * NOTE: when dealing with inner joins, it is appropriate to let a qual clause
  * be evaluated at the lowest level where all the variables it mentions are
@@ -181,7 +214,7 @@ add_missing_rels_to_query(Query *root, Node *jtnode)
  * a rel, thereby forcing them up the join tree to the right level.
  *
  * To ease the calculation of these values, distribute_quals_to_rels() returns
- * the list of Relids involved in its own level of join.  This is just an
+ * the list of base Relids involved in its own level of join.  This is just an
  * internal convenience; no outside callers pay attention to the result.
  */
 Relids
@@ -302,7 +335,7 @@ mark_baserels_for_outer_join(Query *root, Relids rels, Relids outerrels)
 	foreach(relid, rels)
 	{
 		int			relno = lfirsti(relid);
-		RelOptInfo *rel = build_base_rel(root, relno);
+		RelOptInfo *rel = find_base_rel(root, relno);
 
 		/*
 		 * Since we do this bottom-up, any outer-rels previously marked
@@ -370,6 +403,9 @@ distribute_qual_to_rels(Query *root, Node *clause,
 	restrictinfo->right_sortop = InvalidOid;
 	restrictinfo->left_pathkey = NIL;	/* not computable yet */
 	restrictinfo->right_pathkey = NIL;
+	restrictinfo->left_mergescansel = -1;		/* not computed until
+												 * needed */
+	restrictinfo->right_mergescansel = -1;
 	restrictinfo->hashjoinoperator = InvalidOid;
 	restrictinfo->left_bucketsize = -1; /* not computed until needed */
 	restrictinfo->right_bucketsize = -1;
@@ -380,11 +416,41 @@ distribute_qual_to_rels(Query *root, Node *clause,
 	clause_get_relids_vars(clause, &relids, &vars);
 
 	/*
-	 * Cross-check: clause should contain no relids not within its scope.
-	 * Otherwise the parser messed up.
+	 * The clause might contain some join alias vars; if so, we want to
+	 * remove the join otherrelids from relids and add the referent joins'
+	 * scope lists instead (thus ensuring that the clause can be evaluated
+	 * no lower than that join node).  We rely here on the marking done
+	 * earlier by add_base_rels_to_query.
+	 *
+	 * We can combine this step with a cross-check that the clause contains
+	 * no relids not within its scope.	If the first crosscheck succeeds,
+	 * the clause contains no aliases and we needn't look more closely.
 	 */
 	if (!is_subseti(relids, qualscope))
-		elog(ERROR, "JOIN qualification may not refer to other relations");
+	{
+		Relids		newrelids = NIL;
+		List	   *relid;
+
+		foreach(relid, relids)
+		{
+			RelOptInfo *rel = find_other_rel(root, lfirsti(relid));
+
+			if (rel && rel->outerjoinset)
+			{
+				/* this relid is for a join RTE */
+				newrelids = set_unioni(newrelids, rel->outerjoinset);
+			}
+			else
+			{
+				/* this relid is for a true baserel */
+				newrelids = lappendi(newrelids, lfirsti(relid));
+			}
+		}
+		relids = newrelids;
+		/* Now repeat the crosscheck */
+		if (!is_subseti(relids, qualscope))
+			elog(ERROR, "JOIN qualification may not refer to other relations");
+	}
 
 	/*
 	 * If the clause is variable-free, we force it to be evaluated at its
@@ -437,7 +503,7 @@ distribute_qual_to_rels(Query *root, Node *clause,
 		can_be_equijoin = true;
 		foreach(relid, relids)
 		{
-			RelOptInfo *rel = build_base_rel(root, lfirsti(relid));
+			RelOptInfo *rel = find_base_rel(root, lfirsti(relid));
 
 			if (rel->outerjoinset &&
 				!is_subseti(rel->outerjoinset, relids))
@@ -473,7 +539,7 @@ distribute_qual_to_rels(Query *root, Node *clause,
 		 * There is only one relation participating in 'clause', so
 		 * 'clause' is a restriction clause for that relation.
 		 */
-		RelOptInfo *rel = build_base_rel(root, lfirsti(relids));
+		RelOptInfo *rel = find_base_rel(root, lfirsti(relids));
 
 		/*
 		 * Check for a "mergejoinable" clause even though it's not a join
@@ -593,7 +659,7 @@ add_join_info_to_rels(Query *root, RestrictInfo *restrictinfo,
 		 * Find or make the joininfo node for this combination of rels,
 		 * and add the restrictinfo node to it.
 		 */
-		joininfo = find_joininfo_node(build_base_rel(root, cur_relid),
+		joininfo = find_joininfo_node(find_base_rel(root, cur_relid),
 									  unjoined_relids);
 		joininfo->jinfo_restrictinfo = lappend(joininfo->jinfo_restrictinfo,
 											   restrictinfo);
@@ -638,9 +704,6 @@ process_implied_equality(Query *root, Node *item1, Node *item2,
 	 * If both vars belong to same rel, we need to look at that rel's
 	 * baserestrictinfo list.  If different rels, each will have a
 	 * joininfo node for the other, and we can scan either list.
-	 *
-	 * All baserel entries should already exist at this point, so use
-	 * find_base_rel not build_base_rel.
 	 */
 	rel1 = find_base_rel(root, irel1);
 	if (irel1 == irel2)
@@ -678,7 +741,8 @@ process_implied_equality(Query *root, Node *item1, Node *item2,
 	 */
 	ltype = exprType(item1);
 	rtype = exprType(item2);
-	eq_operator = compatible_oper("=", ltype, rtype, true);
+	eq_operator = compatible_oper(makeList1(makeString("=")),
+								  ltype, rtype, true);
 	if (!HeapTupleIsValid(eq_operator))
 	{
 		/*
@@ -687,7 +751,7 @@ process_implied_equality(Query *root, Node *item1, Node *item2,
 		 * datatypes?  NO, because sortkey selection may screw up anyway.
 		 */
 		elog(ERROR, "Unable to identify an equality operator for types '%s' and '%s'",
-			 typeidTypeName(ltype), typeidTypeName(rtype));
+			 format_type_be(ltype), format_type_be(rtype));
 	}
 	pgopform = (Form_pg_operator) GETSTRUCT(eq_operator);
 
@@ -698,14 +762,15 @@ process_implied_equality(Query *root, Node *item1, Node *item2,
 		pgopform->oprrsortop != sortop2 ||
 		pgopform->oprresult != BOOLOID)
 		elog(ERROR, "Equality operator for types '%s' and '%s' should be mergejoinable, but isn't",
-			 typeidTypeName(ltype), typeidTypeName(rtype));
+			 format_type_be(ltype), format_type_be(rtype));
 
 	clause = makeNode(Expr);
 	clause->typeOid = BOOLOID;
 	clause->opType = OP_EXPR;
 	clause->oper = (Node *) makeOper(oprid(eq_operator),		/* opno */
 									 InvalidOid,		/* opid */
-									 BOOLOID);	/* operator result type */
+									 BOOLOID,	/* opresulttype */
+									 false);	/* opretset */
 	clause->args = makeList2(item1, item2);
 
 	ReleaseSysCache(eq_operator);
