@@ -3,12 +3,12 @@
  * parse_expr.c
  *	  handle expressions in parser
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/parser/parse_expr.c,v 1.105 2001/11/12 20:05:24 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/parser/parse_expr.c,v 1.129 2002/09/18 21:35:22 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,6 +17,7 @@
 
 #include "catalog/pg_operator.h"
 #include "catalog/pg_proc.h"
+#include "miscadmin.h"
 #include "nodes/makefuncs.h"
 #include "nodes/params.h"
 #include "parser/analyze.h"
@@ -27,9 +28,9 @@
 #include "parser/parse_func.h"
 #include "parser/parse_oper.h"
 #include "parser/parse_relation.h"
-#include "parser/parse_target.h"
 #include "parser/parse_type.h"
 #include "utils/builtins.h"
+#include "utils/lsyscache.h"
 #include "utils/syscache.h"
 
 
@@ -38,11 +39,8 @@ static int	expr_depth_counter = 0;
 
 bool		Transform_null_equals = false;
 
-static Node *parser_typecast_constant(Value *expr, TypeName *typename);
-static Node *parser_typecast_expression(ParseState *pstate,
-						   Node *expr, TypeName *typename);
-static Node *transformAttr(ParseState *pstate, Attr *att, int precedence);
-static Node *transformIdent(ParseState *pstate, Ident *ident, int precedence);
+static Node *typecast_expression(Node *expr, TypeName *typename);
+static Node *transformColumnRef(ParseState *pstate, ColumnRef *cref);
 static Node *transformIndirection(ParseState *pstate, Node *basenode,
 					 List *indirection);
 
@@ -85,7 +83,7 @@ parse_expr_init(void)
  * input and output of transformExpr; see SubLink for example.
  */
 Node *
-transformExpr(ParseState *pstate, Node *expr, int precedence)
+transformExpr(ParseState *pstate, Node *expr)
 {
 	Node	   *result = NULL;
 
@@ -105,9 +103,38 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 
 	switch (nodeTag(expr))
 	{
-		case T_Attr:
+		case T_ColumnRef:
 			{
-				result = transformAttr(pstate, (Attr *) expr, precedence);
+				result = transformColumnRef(pstate, (ColumnRef *) expr);
+				break;
+			}
+		case T_ParamRef:
+			{
+				ParamRef   *pref = (ParamRef *) expr;
+				int			paramno = pref->number;
+				Oid			paramtyp = param_type(paramno);
+				Param	   *param;
+				List	   *fields;
+
+				if (!OidIsValid(paramtyp))
+					elog(ERROR, "Parameter '$%d' is out of range", paramno);
+				param = makeNode(Param);
+				param->paramkind = PARAM_NUM;
+				param->paramid = (AttrNumber) paramno;
+				param->paramname = "<unnamed>";
+				param->paramtype = paramtyp;
+				result = (Node *) param;
+				/* handle qualification, if any */
+				foreach(fields, pref->fields)
+				{
+					result = ParseFuncOrColumn(pstate,
+											   makeList1(lfirst(fields)),
+											   makeList1(result),
+											   false, false, true);
+				}
+				/* handle subscripts, if any */
+				result = transformIndirection(pstate, result,
+											  pref->indirection);
 				break;
 			}
 		case T_A_Const:
@@ -115,39 +142,36 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 				A_Const    *con = (A_Const *) expr;
 				Value	   *val = &con->val;
 
+				result = (Node *) make_const(val);
 				if (con->typename != NULL)
-					result = parser_typecast_constant(val, con->typename);
-				else
-					result = (Node *) make_const(val);
+					result = typecast_expression(result, con->typename);
 				break;
 			}
-		case T_ParamNo:
+		case T_ExprFieldSelect:
 			{
-				ParamNo    *pno = (ParamNo *) expr;
-				int			paramno = pno->number;
-				Oid			toid = param_type(paramno);
-				Param	   *param = makeNode(Param);
+				ExprFieldSelect *efs = (ExprFieldSelect *) expr;
+				List	   *fields;
 
-				if (!OidIsValid(toid))
-					elog(ERROR, "Parameter '$%d' is out of range", paramno);
-				param->paramkind = PARAM_NUM;
-				param->paramid = (AttrNumber) paramno;
-				param->paramname = "<unnamed>";
-				param->paramtype = toid;
-				result = transformIndirection(pstate, (Node *) param,
-											  pno->indirection);
-				/* cope with typecast applied to param */
-				if (pno->typename != NULL)
-					result = parser_typecast_expression(pstate, result,
-														pno->typename);
+				result = transformExpr(pstate, efs->arg);
+				/* handle qualification, if any */
+				foreach(fields, efs->fields)
+				{
+					result = ParseFuncOrColumn(pstate,
+											   makeList1(lfirst(fields)),
+											   makeList1(result),
+											   false, false, true);
+				}
+				/* handle subscripts, if any */
+				result = transformIndirection(pstate, result,
+											  efs->indirection);
 				break;
 			}
 		case T_TypeCast:
 			{
 				TypeCast   *tc = (TypeCast *) expr;
-				Node	   *arg = transformExpr(pstate, tc->arg, precedence);
+				Node	   *arg = transformExpr(pstate, tc->arg);
 
-				result = parser_typecast_expression(pstate, arg, tc->typename);
+				result = typecast_expression(arg, tc->typename);
 				break;
 			}
 		case T_A_Expr:
@@ -165,7 +189,8 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 							 * into IS NULL exprs.
 							 */
 							if (Transform_null_equals &&
-								strcmp(a->opname, "=") == 0 &&
+								length(a->name) == 1 &&
+							 strcmp(strVal(lfirst(a->name)), "=") == 0 &&
 								(exprIsNullConstant(a->lexpr) ||
 								 exprIsNullConstant(a->rexpr)))
 							{
@@ -179,19 +204,16 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 									n->arg = a->lexpr;
 
 								result = transformExpr(pstate,
-													   (Node *) n,
-													   precedence);
+													   (Node *) n);
 							}
 							else
 							{
 								Node	   *lexpr = transformExpr(pstate,
-																a->lexpr,
-															 precedence);
+															   a->lexpr);
 								Node	   *rexpr = transformExpr(pstate,
-																a->rexpr,
-															 precedence);
+															   a->rexpr);
 
-								result = (Node *) make_op(a->opname,
+								result = (Node *) make_op(a->name,
 														  lexpr,
 														  rexpr);
 							}
@@ -200,22 +222,13 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					case AND:
 						{
 							Node	   *lexpr = transformExpr(pstate,
-															  a->lexpr,
-															  precedence);
+															  a->lexpr);
 							Node	   *rexpr = transformExpr(pstate,
-															  a->rexpr,
-															  precedence);
+															  a->rexpr);
 							Expr	   *expr = makeNode(Expr);
 
-							if (!coerce_to_boolean(pstate, &lexpr))
-								elog(ERROR, "left-hand side of AND is type '%s', not '%s'",
-									 format_type_be(exprType(lexpr)),
-									 format_type_be(BOOLOID));
-
-							if (!coerce_to_boolean(pstate, &rexpr))
-								elog(ERROR, "right-hand side of AND is type '%s', not '%s'",
-									 format_type_be(exprType(rexpr)),
-									 format_type_be(BOOLOID));
+							lexpr = coerce_to_boolean(lexpr, "AND");
+							rexpr = coerce_to_boolean(rexpr, "AND");
 
 							expr->typeOid = BOOLOID;
 							expr->opType = AND_EXPR;
@@ -226,22 +239,13 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					case OR:
 						{
 							Node	   *lexpr = transformExpr(pstate,
-															  a->lexpr,
-															  precedence);
+															  a->lexpr);
 							Node	   *rexpr = transformExpr(pstate,
-															  a->rexpr,
-															  precedence);
+															  a->rexpr);
 							Expr	   *expr = makeNode(Expr);
 
-							if (!coerce_to_boolean(pstate, &lexpr))
-								elog(ERROR, "left-hand side of OR is type '%s', not '%s'",
-									 format_type_be(exprType(lexpr)),
-									 format_type_be(BOOLOID));
-
-							if (!coerce_to_boolean(pstate, &rexpr))
-								elog(ERROR, "right-hand side of OR is type '%s', not '%s'",
-									 format_type_be(exprType(rexpr)),
-									 format_type_be(BOOLOID));
+							lexpr = coerce_to_boolean(lexpr, "OR");
+							rexpr = coerce_to_boolean(rexpr, "OR");
 
 							expr->typeOid = BOOLOID;
 							expr->opType = OR_EXPR;
@@ -252,14 +256,10 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					case NOT:
 						{
 							Node	   *rexpr = transformExpr(pstate,
-															  a->rexpr,
-															  precedence);
+															  a->rexpr);
 							Expr	   *expr = makeNode(Expr);
 
-							if (!coerce_to_boolean(pstate, &rexpr))
-								elog(ERROR, "argument to NOT is type '%s', not '%s'",
-									 format_type_be(exprType(rexpr)),
-									 format_type_be(BOOLOID));
+							rexpr = coerce_to_boolean(rexpr, "NOT");
 
 							expr->typeOid = BOOLOID;
 							expr->opType = NOT_EXPR;
@@ -267,14 +267,62 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 							result = (Node *) expr;
 						}
 						break;
+					case DISTINCT:
+						{
+							Node	   *lexpr = transformExpr(pstate,
+															  a->lexpr);
+							Node	   *rexpr = transformExpr(pstate,
+															  a->rexpr);
+
+							result = (Node *) make_op(a->name,
+													  lexpr,
+													  rexpr);
+							((Expr *) result)->opType = DISTINCT_EXPR;
+						}
+						break;
+					case OF:
+						{
+							List	   *telem;
+							A_Const    *n;
+							Oid			ltype,
+										rtype;
+							bool		matched = FALSE;
+
+							/*
+							 * Checking an expression for match to type.
+							 * Will result in a boolean constant node.
+							 */
+							Node	   *lexpr = transformExpr(pstate,
+															  a->lexpr);
+
+							ltype = exprType(lexpr);
+							foreach(telem, (List *) a->rexpr)
+							{
+								rtype = LookupTypeName(lfirst(telem));
+								matched = (rtype == ltype);
+								if (matched)
+									break;
+							}
+
+							/*
+							 * Expect two forms: equals or not equals.
+							 * Flip the sense of the result for not
+							 * equals.
+							 */
+							if (strcmp(strVal(lfirst(a->name)), "!=") == 0)
+								matched = (!matched);
+
+							n = makeNode(A_Const);
+							n->val.type = T_String;
+							n->val.val.str = (matched ? "t" : "f");
+							n->typename = SystemTypeName("bool");
+
+							result = transformExpr(pstate, (Node *) n);
+						}
+						break;
 				}
 				break;
 			}
-		case T_Ident:
-			{
-				result = transformIdent(pstate, (Ident *) expr, precedence);
-				break;
-			}
 		case T_FuncCall:
 			{
 				FuncCall   *fn = (FuncCall *) expr;
@@ -283,14 +331,13 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 				/* transform the list of arguments */
 				foreach(args, fn->args)
 					lfirst(args) = transformExpr(pstate,
-												 (Node *) lfirst(args),
-												 precedence);
+												 (Node *) lfirst(args));
 				result = ParseFuncOrColumn(pstate,
 										   fn->funcname,
 										   fn->args,
 										   fn->agg_star,
 										   fn->agg_distinct,
-										   precedence);
+										   false);
 				break;
 			}
 		case T_SubLink:
@@ -353,22 +400,23 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					/* ALL, ANY, or MULTIEXPR: generate operator list */
 					List	   *left_list = sublink->lefthand;
 					List	   *right_list = qtree->targetList;
-					char	   *op;
+					List	   *op;
+					char	   *opname;
 					List	   *elist;
 
 					foreach(elist, left_list)
-						lfirst(elist) = transformExpr(pstate, lfirst(elist),
-													  precedence);
+						lfirst(elist) = transformExpr(pstate, lfirst(elist));
 
 					Assert(IsA(sublink->oper, A_Expr));
-					op = ((A_Expr *) sublink->oper)->opname;
+					op = ((A_Expr *) sublink->oper)->name;
+					opname = strVal(llast(op));
 					sublink->oper = NIL;
 
 					/* Combining operators other than =/<> is dubious... */
 					if (length(left_list) != 1 &&
-						strcmp(op, "=") != 0 && strcmp(op, "<>") != 0)
-						elog(ERROR, "Row comparison cannot use '%s'",
-							 op);
+					strcmp(opname, "=") != 0 && strcmp(opname, "<>") != 0)
+						elog(ERROR, "Row comparison cannot use operator %s",
+							 opname);
 
 					/*
 					 * Scan subquery's targetlist to find values that will
@@ -406,14 +454,20 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 						opform = (Form_pg_operator) GETSTRUCT(optup);
 
 						if (opform->oprresult != BOOLOID)
-							elog(ERROR, "'%s' result type of '%s' must return '%s'"
+							elog(ERROR, "%s has result type of %s, but must return %s"
 								 " to be used with quantified predicate subquery",
-								 op, typeidTypeName(opform->oprresult),
-								 typeidTypeName(BOOLOID));
+							   opname, format_type_be(opform->oprresult),
+								 format_type_be(BOOLOID));
+
+						if (get_func_retset(opform->oprcode))
+							elog(ERROR, "%s must not return a set"
+								 " to be used with quantified predicate subquery",
+								 opname);
 
 						newop = makeOper(oprid(optup),	/* opno */
 										 InvalidOid,	/* opid */
-										 opform->oprresult);
+										 opform->oprresult,
+										 false);
 						sublink->oper = lappend(sublink->oper, newop);
 						ReleaseSysCache(optup);
 					}
@@ -447,18 +501,12 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					if (c->arg != NULL)
 					{
 						/* shorthand form was specified, so expand... */
-						A_Expr	   *a = makeNode(A_Expr);
-
-						a->oper = OP;
-						a->opname = "=";
-						a->lexpr = c->arg;
-						a->rexpr = warg;
-						warg = (Node *) a;
+						warg = (Node *) makeSimpleA_Expr(OP, "=",
+														 c->arg, warg);
 					}
-					neww->expr = transformExpr(pstate, warg, precedence);
+					neww->expr = transformExpr(pstate, warg);
 
-					if (!coerce_to_boolean(pstate, &neww->expr))
-						elog(ERROR, "WHEN clause must have a boolean result");
+					neww->expr = coerce_to_boolean(neww->expr, "CASE/WHEN");
 
 					/*
 					 * result is NULL for NULLIF() construct - thomas
@@ -472,7 +520,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 						n->val.type = T_Null;
 						warg = (Node *) n;
 					}
-					neww->result = transformExpr(pstate, warg, precedence);
+					neww->result = transformExpr(pstate, warg);
 
 					newargs = lappend(newargs, neww);
 					typeids = lappendi(typeids, exprType(neww->result));
@@ -496,7 +544,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 					n->val.type = T_Null;
 					defresult = (Node *) n;
 				}
-				newc->defresult = transformExpr(pstate, defresult, precedence);
+				newc->defresult = transformExpr(pstate, defresult);
 
 				/*
 				 * Note: default result is considered the most significant
@@ -510,8 +558,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 				newc->casetype = ptype;
 
 				/* Convert default result clause, if necessary */
-				newc->defresult = coerce_to_common_type(pstate,
-														newc->defresult,
+				newc->defresult = coerce_to_common_type(newc->defresult,
 														ptype,
 														"CASE/ELSE");
 
@@ -520,8 +567,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 				{
 					CaseWhen   *w = (CaseWhen *) lfirst(args);
 
-					w->result = coerce_to_common_type(pstate,
-													  w->result,
+					w->result = coerce_to_common_type(w->result,
 													  ptype,
 													  "CASE/WHEN");
 				}
@@ -534,7 +580,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 			{
 				NullTest   *n = (NullTest *) expr;
 
-				n->arg = transformExpr(pstate, n->arg, precedence);
+				n->arg = transformExpr(pstate, n->arg);
 				/* the argument can be any type, so don't coerce it */
 				result = expr;
 				break;
@@ -543,54 +589,50 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 		case T_BooleanTest:
 			{
 				BooleanTest *b = (BooleanTest *) expr;
+				const char *clausename;
 
-				b->arg = transformExpr(pstate, b->arg, precedence);
-
-				if (!coerce_to_boolean(pstate, &b->arg))
+				switch (b->booltesttype)
 				{
-					const char *clausename;
-
-					switch (b->booltesttype)
-					{
-						case IS_TRUE:
-							clausename = "IS TRUE";
-							break;
-						case IS_NOT_TRUE:
-							clausename = "IS NOT TRUE";
-							break;
-						case IS_FALSE:
-							clausename = "IS FALSE";
-							break;
-						case IS_NOT_FALSE:
-							clausename = "IS NOT FALSE";
-							break;
-						case IS_UNKNOWN:
-							clausename = "IS UNKNOWN";
-							break;
-						case IS_NOT_UNKNOWN:
-							clausename = "IS NOT UNKNOWN";
-							break;
-						default:
-							elog(ERROR, "transformExpr: unexpected booltesttype %d",
-								 (int) b->booltesttype);
-							clausename = NULL;	/* keep compiler quiet */
-					}
-
-					elog(ERROR, "Argument of %s must be boolean",
-						 clausename);
+					case IS_TRUE:
+						clausename = "IS TRUE";
+						break;
+					case IS_NOT_TRUE:
+						clausename = "IS NOT TRUE";
+						break;
+					case IS_FALSE:
+						clausename = "IS FALSE";
+						break;
+					case IS_NOT_FALSE:
+						clausename = "IS NOT FALSE";
+						break;
+					case IS_UNKNOWN:
+						clausename = "IS UNKNOWN";
+						break;
+					case IS_NOT_UNKNOWN:
+						clausename = "IS NOT UNKNOWN";
+						break;
+					default:
+						elog(ERROR, "transformExpr: unexpected booltesttype %d",
+							 (int) b->booltesttype);
+						clausename = NULL;		/* keep compiler quiet */
 				}
+
+				b->arg = transformExpr(pstate, b->arg);
+
+				b->arg = coerce_to_boolean(b->arg, clausename);
+
 				result = expr;
 				break;
 			}
 
-			/*
+			/*********************************************
 			 * Quietly accept node types that may be presented when we are
 			 * called on an already-transformed tree.
 			 *
 			 * Do any other node types need to be accepted?  For now we are
 			 * taking a conservative approach, and only accepting node
 			 * types that are demonstrably necessary to accept.
-			 */
+			 *********************************************/
 		case T_Expr:
 		case T_Var:
 		case T_Const:
@@ -599,6 +641,7 @@ transformExpr(ParseState *pstate, Node *expr, int precedence)
 		case T_ArrayRef:
 		case T_FieldSelect:
 		case T_RelabelType:
+		case T_ConstraintTest:
 			{
 				result = (Node *) expr;
 				break;
@@ -622,52 +665,200 @@ transformIndirection(ParseState *pstate, Node *basenode, List *indirection)
 	if (indirection == NIL)
 		return basenode;
 	return (Node *) transformArraySubscripts(pstate,
-											 basenode, exprType(basenode),
-											 indirection, false, NULL);
+											 basenode,
+											 exprType(basenode),
+											 exprTypmod(basenode),
+											 indirection,
+											 false,
+											 NULL);
 }
 
 static Node *
-transformAttr(ParseState *pstate, Attr *att, int precedence)
+transformColumnRef(ParseState *pstate, ColumnRef *cref)
 {
-	Node	   *basenode;
+	int			numnames = length(cref->fields);
+	Node	   *node;
+	RangeVar   *rv;
 
-	basenode = ParseNestedFuncOrColumn(pstate, att, precedence);
-	return transformIndirection(pstate, basenode, att->indirection);
-}
-
-static Node *
-transformIdent(ParseState *pstate, Ident *ident, int precedence)
-{
-	Node	   *result = NULL;
-	int			sublevels_up;
-
-	/*
-	 * try to find the ident as a relation ... but not if subscripts
-	 * appear
+	/*----------
+	 * The allowed syntaxes are:
+	 *
+	 * A		First try to resolve as unqualified column name;
+	 *			if no luck, try to resolve as unqual. table name (A.*).
+	 * A.B		A is an unqual. table name; B is either a
+	 *			column or function name (trying column name first).
+	 * A.B.C	schema A, table B, col or func name C.
+	 * A.B.C.D	catalog A, schema B, table C, col or func D.
+	 * A.*		A is an unqual. table name; means whole-row value.
+	 * A.B.*	whole-row value of table B in schema A.
+	 * A.B.C.*	whole-row value of table C in schema B in catalog A.
+	 *
+	 * We do not need to cope with bare "*"; that will only be accepted by
+	 * the grammar at the top level of a SELECT list, and transformTargetList
+	 * will take care of it before it ever gets here.
+	 *
+	 * Currently, if a catalog name is given then it must equal the current
+	 * database name; we check it here and then discard it.
+	 *
+	 * For whole-row references, the result is an untransformed RangeVar,
+	 * which will work as the argument to a function call, but not in any
+	 * other context at present.  (We could instead coerce to a whole-row Var,
+	 * but that will fail for subselect and join RTEs, because there is no
+	 * pg_type entry for their rowtypes.)
+	 *----------
 	 */
-	if (ident->indirection == NIL &&
-	 refnameRangeOrJoinEntry(pstate, ident->name, &sublevels_up) != NULL)
+	switch (numnames)
 	{
-		ident->isRel = TRUE;
-		result = (Node *) ident;
+		case 1:
+			{
+				char	   *name = strVal(lfirst(cref->fields));
+
+				/* Try to identify as an unqualified column */
+				node = colnameToVar(pstate, name);
+
+				if (node == NULL)
+				{
+					/*
+					 * Not known as a column of any range-table entry, so
+					 * try to find the name as a relation ... but not if
+					 * subscripts appear.  Note also that only relations
+					 * already entered into the rangetable will be
+					 * recognized.
+					 *
+					 * This is a hack for backwards compatibility with
+					 * PostQUEL- inspired syntax.  The preferred form now
+					 * is "rel.*".
+					 */
+					int			levels_up;
+
+					if (cref->indirection == NIL &&
+						refnameRangeTblEntry(pstate, NULL, name,
+											 &levels_up) != NULL)
+					{
+						rv = makeNode(RangeVar);
+						rv->relname = name;
+						rv->inhOpt = INH_DEFAULT;
+						node = (Node *) rv;
+					}
+					else
+						elog(ERROR, "Attribute \"%s\" not found", name);
+				}
+				break;
+			}
+		case 2:
+			{
+				char	   *name1 = strVal(lfirst(cref->fields));
+				char	   *name2 = strVal(lsecond(cref->fields));
+
+				/* Whole-row reference? */
+				if (strcmp(name2, "*") == 0)
+				{
+					rv = makeNode(RangeVar);
+					rv->relname = name1;
+					rv->inhOpt = INH_DEFAULT;
+					node = (Node *) rv;
+					break;
+				}
+
+				/* Try to identify as a once-qualified column */
+				node = qualifiedNameToVar(pstate, NULL, name1, name2, true);
+				if (node == NULL)
+				{
+					/*
+					 * Not known as a column of any range-table entry, so
+					 * try it as a function call.  Here, we will create an
+					 * implicit RTE for tables not already entered.
+					 */
+					rv = makeNode(RangeVar);
+					rv->relname = name1;
+					rv->inhOpt = INH_DEFAULT;
+					node = ParseFuncOrColumn(pstate,
+											 makeList1(makeString(name2)),
+											 makeList1(rv),
+											 false, false, true);
+				}
+				break;
+			}
+		case 3:
+			{
+				char	   *name1 = strVal(lfirst(cref->fields));
+				char	   *name2 = strVal(lsecond(cref->fields));
+				char	   *name3 = strVal(lfirst(lnext(lnext(cref->fields))));
+
+				/* Whole-row reference? */
+				if (strcmp(name3, "*") == 0)
+				{
+					rv = makeNode(RangeVar);
+					rv->schemaname = name1;
+					rv->relname = name2;
+					rv->inhOpt = INH_DEFAULT;
+					node = (Node *) rv;
+					break;
+				}
+
+				/* Try to identify as a twice-qualified column */
+				node = qualifiedNameToVar(pstate, name1, name2, name3, true);
+				if (node == NULL)
+				{
+					/* Try it as a function call */
+					rv = makeNode(RangeVar);
+					rv->schemaname = name1;
+					rv->relname = name2;
+					rv->inhOpt = INH_DEFAULT;
+					node = ParseFuncOrColumn(pstate,
+											 makeList1(makeString(name3)),
+											 makeList1(rv),
+											 false, false, true);
+				}
+				break;
+			}
+		case 4:
+			{
+				char	   *name1 = strVal(lfirst(cref->fields));
+				char	   *name2 = strVal(lsecond(cref->fields));
+				char	   *name3 = strVal(lfirst(lnext(lnext(cref->fields))));
+				char	   *name4 = strVal(lfirst(lnext(lnext(lnext(cref->fields)))));
+
+				/*
+				 * We check the catalog name and then ignore it.
+				 */
+				if (strcmp(name1, DatabaseName) != 0)
+					elog(ERROR, "Cross-database references are not implemented");
+
+				/* Whole-row reference? */
+				if (strcmp(name4, "*") == 0)
+				{
+					rv = makeNode(RangeVar);
+					rv->schemaname = name2;
+					rv->relname = name3;
+					rv->inhOpt = INH_DEFAULT;
+					node = (Node *) rv;
+					break;
+				}
+
+				/* Try to identify as a twice-qualified column */
+				node = qualifiedNameToVar(pstate, name2, name3, name4, true);
+				if (node == NULL)
+				{
+					/* Try it as a function call */
+					rv = makeNode(RangeVar);
+					rv->schemaname = name2;
+					rv->relname = name3;
+					rv->inhOpt = INH_DEFAULT;
+					node = ParseFuncOrColumn(pstate,
+											 makeList1(makeString(name4)),
+											 makeList1(rv),
+											 false, false, true);
+				}
+				break;
+			}
+		default:
+			elog(ERROR, "Invalid qualified name syntax (too many names)");
+			node = NULL;		/* keep compiler quiet */
+			break;
 	}
 
-	if (result == NULL || precedence == EXPR_COLUMN_FIRST)
-	{
-		/* try to find the ident as a column */
-		Node	   *var = colnameToVar(pstate, ident->name);
-
-		if (var != NULL)
-		{
-			ident->isRel = FALSE;
-			result = transformIndirection(pstate, var, ident->indirection);
-		}
-	}
-
-	if (result == NULL)
-		elog(ERROR, "Attribute '%s' not found", ident->name);
-
-	return result;
+	return transformIndirection(pstate, node, cref->indirection);
 }
 
 /*
@@ -684,12 +875,6 @@ exprType(Node *expr)
 
 	switch (nodeTag(expr))
 	{
-		case T_Func:
-			type = ((Func *) expr)->functype;
-			break;
-		case T_Iter:
-			type = ((Iter *) expr)->itertype;
-			break;
 		case T_Var:
 			type = ((Var *) expr)->vartype;
 			break;
@@ -700,7 +885,7 @@ exprType(Node *expr)
 			type = ((Const *) expr)->consttype;
 			break;
 		case T_ArrayRef:
-			type = ((ArrayRef *) expr)->refelemtype;
+			type = ((ArrayRef *) expr)->refrestype;
 			break;
 		case T_Aggref:
 			type = ((Aggref *) expr)->aggtype;
@@ -725,7 +910,7 @@ exprType(Node *expr)
 					TargetEntry *tent;
 
 					if (!qtree || !IsA(qtree, Query))
-						elog(ERROR, "Cannot get type for untransformed sublink");
+						elog(ERROR, "exprType: Cannot get type for untransformed sublink");
 					tent = (TargetEntry *) lfirst(qtree->targetList);
 					type = tent->resdom->restype;
 				}
@@ -748,12 +933,11 @@ exprType(Node *expr)
 		case T_BooleanTest:
 			type = BOOLOID;
 			break;
-		case T_Ident:
-			/* XXX is this right? */
-			type = UNKNOWNOID;
+		case T_ConstraintTest:
+			type = exprType(((ConstraintTest *) expr)->arg);
 			break;
 		default:
-			elog(ERROR, "Do not know how to get type for %d node",
+			elog(ERROR, "exprType: Do not know how to get type for %d node",
 				 nodeTag(expr));
 			break;
 	}
@@ -802,10 +986,8 @@ exprTypmod(Node *expr)
 			break;
 		case T_FieldSelect:
 			return ((FieldSelect *) expr)->resulttypmod;
-			break;
 		case T_RelabelType:
 			return ((RelabelType *) expr)->resulttypmod;
-			break;
 		case T_CaseExpr:
 			{
 				/*
@@ -837,6 +1019,9 @@ exprTypmod(Node *expr)
 				return typmod;
 			}
 			break;
+		case T_ConstraintTest:
+			return exprTypmod(((ConstraintTest *) expr)->arg);
+
 		default:
 			break;
 	}
@@ -850,23 +1035,13 @@ exprTypmod(Node *expr)
  *
  * If coercedTypmod is not NULL, the typmod is stored there if the expression
  * is a length-coercion function, else -1 is stored there.
- *
- * We assume that a two-argument function named for a datatype, whose
- * output and first argument types are that datatype, and whose second
- * input is an int32 constant, represents a forced length coercion.
- *
- * XXX It'd be better if the parsetree retained some explicit indication
- * of the coercion, so we didn't need these heuristics.
  */
 bool
 exprIsLengthCoercion(Node *expr, int32 *coercedTypmod)
 {
 	Func	   *func;
+	int			nargs;
 	Const	   *second_arg;
-	HeapTuple	procTuple;
-	HeapTuple	typeTuple;
-	Form_pg_proc procStruct;
-	Form_pg_type typeStruct;
 
 	if (coercedTypmod != NULL)
 		*coercedTypmod = -1;	/* default result on failure */
@@ -880,190 +1055,61 @@ exprIsLengthCoercion(Node *expr, int32 *coercedTypmod)
 	Assert(IsA(func, Func));
 
 	/*
-	 * If it's not a two-argument function with the second argument being
-	 * an int4 constant, it can't have been created from a length
-	 * coercion.
+	 * If it didn't come from a coercion context, reject.
 	 */
-	if (length(((Expr *) expr)->args) != 2)
+	if (func->funcformat != COERCE_EXPLICIT_CAST &&
+		func->funcformat != COERCE_IMPLICIT_CAST)
 		return false;
+
+	/*
+	 * If it's not a two-argument or three-argument function with the second
+	 * argument being an int4 constant, it can't have been created from a
+	 * length coercion (it must be a type coercion, instead).
+	 */
+	nargs = length(((Expr *) expr)->args);
+	if (nargs < 2 || nargs > 3)
+		return false;
+
 	second_arg = (Const *) lsecond(((Expr *) expr)->args);
 	if (!IsA(second_arg, Const) ||
 		second_arg->consttype != INT4OID ||
 		second_arg->constisnull)
 		return false;
 
-	/*
-	 * Lookup the function in pg_proc
-	 */
-	procTuple = SearchSysCache(PROCOID,
-							   ObjectIdGetDatum(func->funcid),
-							   0, 0, 0);
-	if (!HeapTupleIsValid(procTuple))
-		elog(ERROR, "cache lookup for proc %u failed", func->funcid);
-	procStruct = (Form_pg_proc) GETSTRUCT(procTuple);
-
-	/*
-	 * It must be a function with two arguments where the first is of the
-	 * same type as the return value and the second is an int4. Also, just
-	 * to be sure, check return type agrees with expr node.
-	 */
-	if (procStruct->pronargs != 2 ||
-		procStruct->prorettype != procStruct->proargtypes[0] ||
-		procStruct->proargtypes[1] != INT4OID ||
-		procStruct->prorettype != ((Expr *) expr)->typeOid)
-	{
-		ReleaseSysCache(procTuple);
-		return false;
-	}
-
-	/*
-	 * Furthermore, the name of the function must be the same as the
-	 * argument/result type's name.
-	 */
-	typeTuple = SearchSysCache(TYPEOID,
-							   ObjectIdGetDatum(procStruct->prorettype),
-							   0, 0, 0);
-	if (!HeapTupleIsValid(typeTuple))
-		elog(ERROR, "cache lookup for type %u failed",
-			 procStruct->prorettype);
-	typeStruct = (Form_pg_type) GETSTRUCT(typeTuple);
-	if (strncmp(NameStr(procStruct->proname),
-				NameStr(typeStruct->typname),
-				NAMEDATALEN) != 0)
-	{
-		ReleaseSysCache(procTuple);
-		ReleaseSysCache(typeTuple);
-		return false;
-	}
-
 	/*
 	 * OK, it is indeed a length-coercion function.
 	 */
 	if (coercedTypmod != NULL)
 		*coercedTypmod = DatumGetInt32(second_arg->constvalue);
 
-	ReleaseSysCache(procTuple);
-	ReleaseSysCache(typeTuple);
 	return true;
 }
 
 /*
- * Produce an appropriate Const node from a constant value produced
- * by the parser and an explicit type name to cast to.
- */
-static Node *
-parser_typecast_constant(Value *expr, TypeName *typename)
-{
-	Type		tp;
-	Datum		datum;
-	Const	   *con;
-	char	   *const_string = NULL;
-	bool		string_palloced = false;
-	bool		isNull = false;
-
-	tp = typenameType(TypeNameToInternalName(typename));
-
-	switch (nodeTag(expr))
-	{
-		case T_Integer:
-			const_string = DatumGetCString(DirectFunctionCall1(int4out,
-										 Int32GetDatum(expr->val.ival)));
-			string_palloced = true;
-			break;
-		case T_Float:
-		case T_String:
-		case T_BitString:
-			const_string = expr->val.str;
-			break;
-		case T_Null:
-			isNull = true;
-			break;
-		default:
-			elog(ERROR, "Cannot cast this expression to type '%s'",
-				 typeTypeName(tp));
-	}
-
-	if (isNull)
-		datum = (Datum) NULL;
-	else
-		datum = stringTypeDatum(tp, const_string, typename->typmod);
-
-	con = makeConst(typeTypeId(tp),
-					typeLen(tp),
-					datum,
-					isNull,
-					typeByVal(tp),
-					false,		/* not a set */
-					true /* is cast */ );
-
-	if (string_palloced)
-		pfree(const_string);
-
-	ReleaseSysCache(tp);
-
-	return (Node *) con;
-}
-
-/*
- * Handle an explicit CAST applied to a non-constant expression.
- * (Actually, this works for constants too, but gram.y won't generate
- * a TypeCast node if the argument is just a constant.)
+ * Handle an explicit CAST construct.
  *
  * The given expr has already been transformed, but we need to lookup
  * the type name and then apply any necessary coercion function(s).
  */
 static Node *
-parser_typecast_expression(ParseState *pstate,
-						   Node *expr, TypeName *typename)
+typecast_expression(Node *expr, TypeName *typename)
 {
 	Oid			inputType = exprType(expr);
 	Oid			targetType;
 
-	targetType = typenameTypeId(TypeNameToInternalName(typename));
+	targetType = typenameTypeId(typename);
 
 	if (inputType == InvalidOid)
 		return expr;			/* do nothing if NULL input */
 
-	if (inputType != targetType)
-	{
-		expr = CoerceTargetExpr(pstate, expr, inputType,
-								targetType, typename->typmod);
-		if (expr == NULL)
-			elog(ERROR, "Cannot cast type '%s' to '%s'",
-				 format_type_be(inputType),
-				 format_type_be(targetType));
-	}
-
-	/*
-	 * If the target is a fixed-length type, it may need a length coercion
-	 * as well as a type coercion.
-	 */
-	expr = coerce_type_typmod(pstate, expr,
-							  targetType, typename->typmod);
+	expr = coerce_to_target_type(expr, inputType,
+								 targetType, typename->typmod,
+								 COERCION_EXPLICIT,
+								 COERCE_EXPLICIT_CAST);
+	if (expr == NULL)
+		elog(ERROR, "Cannot cast type %s to %s",
+			 format_type_be(inputType),
+			 format_type_be(targetType));
 
 	return expr;
 }
-
-/*
- * Given a TypeName node as returned by the grammar, generate the internal
- * name of the corresponding type.	Note this does NOT check if the type
- * exists or not.
- */
-char *
-TypeNameToInternalName(TypeName *typename)
-{
-	Assert(typename->attrname == NULL);
-	if (typename->arrayBounds != NIL)
-	{
-		/*
-		 * By convention, the name of an array type is the name of its
-		 * element type with "_" prepended.
-		 */
-		char	   *arrayname = palloc(strlen(typename->name) + 2);
-
-		sprintf(arrayname, "_%s", typename->name);
-		return arrayname;
-	}
-	else
-		return typename->name;
-}
