@@ -24,10 +24,10 @@
 # template0 is made just by copying the completed template1.
 #
 #
-# Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+# Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
 # Portions Copyright (c) 1994, Regents of the University of California
 #
-# $Header: /cvsroot/pgsql/src/bin/initdb/Attic/initdb.sh,v 1.142 2001/11/25 22:19:30 petere Exp $
+# $Header: /cvsroot/pgsql-server/src/bin/initdb/initdb.sh,v 1.176.2.1 2002/11/25 21:42:11 momjian Exp $
 #
 #-------------------------------------------------------------------------
 
@@ -51,6 +51,26 @@ exit_nicely(){
     exit 1
 }
 
+pg_getlocale(){
+    arg=$1
+    unset ret
+
+    for var in "PGLC_$arg" PGLOCALE LC_ALL "LC_$arg" LANG; do
+        varset=`eval echo '${'"$var"'+set}'`
+        varval=`eval echo '$'"$var"`
+        if test "$varset" = set; then
+            ret=$varval
+            break
+        fi
+    done
+
+    if test "${ret+set}" != set; then
+        ret=C
+    fi
+
+    echo "$ret"
+}
+
 
 CMDNAME=`basename $0`
 
@@ -60,8 +80,6 @@ bindir='@bindir@'
 # Note that "datadir" is not the directory we're initializing, it's
 # merely how Autoconf names PREFIX/share.
 datadir='@datadir@'
-# as set by configure --enable-multibyte[=XXX].
-MULTIBYTE='@MULTIBYTE@'
 
 
 # Check for echo -n vs echo \c
@@ -170,7 +188,7 @@ fi
 # COMMAND LINE OPTIONS
 
 # 0 is the default (non-)encoding
-MULTIBYTEID=0
+ENCODINGID=0
 
 # Set defaults:
 debug=
@@ -243,19 +261,69 @@ do
 # The encoding of the template1 database. Defaults to what you chose
 # at configure time. (see above)
         --encoding|-E)
-                MULTIBYTE="$2"
+                ENCODING="$2"
                 shift;;
         --encoding=*)
-                MULTIBYTE=`echo $1 | sed 's/^--encoding=//'`
+                ENCODING=`echo $1 | sed 's/^--encoding=//'`
                 ;;
         -E*)
-                MULTIBYTE=`echo $1 | sed 's/^-E//'`
+                ENCODING=`echo $1 | sed 's/^-E//'`
                 ;;
+# Locale flags
+        --locale)
+                PGLOCALE="$2"
+                shift;;
+        --locale=*)
+                PGLOCALE=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --no-locale)
+                PGLOCALE=C
+                ;;
+
+        --lc-collate)
+                PGLC_COLLATE=$2
+                shift;;
+        --lc-collate=*)
+                PGLC_COLLATE=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --lc-ctype)
+                PGLC_CTYPE=$2
+                shift;;
+        --lc-ctype=*)
+                PGLC_CTYPE=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --lc-messages)
+                PGLC_MESSAGES=$2
+                shift;;
+        --lc-messages=*)
+                PGLC_MESSAGES=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --lc-monetary)
+                PGLC_MONETARY=$2
+                shift;;
+        --lc-monetary=*)
+                PGLC_MONETARY=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --lc-numeric)
+                PGLC_NUMERIC=$2
+                shift;;
+        --lc-numeric=*)
+                PGLC_NUMERIC=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+        --lc-time)
+                PGLC_TIME=$2
+                shift;;
+        --lc-time=*)
+                PGLC_TIME=`echo $1 | sed 's/^[^=]*=//'`
+                ;;
+
 	-*)
 		echo "$CMDNAME: invalid option: $1"
 		echo "Try '$CMDNAME --help' for more information."
 		exit 1
 		;;
+
+# Non-option argument specifies data directory
         *)
                 PGDATA=$1
                 ;;
@@ -267,44 +335,53 @@ if [ "$usage" ]; then
     echo "$CMDNAME initializes a PostgreSQL database cluster."
     echo
     echo "Usage:"
-    echo "  $CMDNAME [options] datadir"
+    echo "  $CMDNAME [OPTION]... [DATADIR]"
     echo
     echo "Options:"
-    echo " [-D, --pgdata] DATADIR       Location for this database cluster"
-    echo "  -W, --pwprompt              Prompt for a password for the new superuser"
-    if [ -n "$MULTIBYTE" ] ; then 
-        echo "  -E, --encoding ENCODING     Set default encoding for new databases"
-    fi
-    echo "  -U, --username NAME         Database superuser name"
+    echo " [-D, --pgdata=]DATADIR     location for this database cluster"
+    echo "  -E, --encoding=ENCODING   set default encoding for new databases"
+    echo "  --locale=LOCALE           initialize database cluster with given locale"
+    echo "  --lc-collate, --lc-ctype, --lc-messages=LOCALE"
+    echo "  --lc-monetary, --lc-numeric, --lc-time=LOCALE"
+    echo "                            initialize database cluster with given locale"
+    echo "                            in the respective category (default taken from"
+    echo "                            environment)"
+    echo "  --no-locale               equivalent to --locale=C"
+    echo "  -U, --username=NAME       database superuser name"
+    echo "  -W, --pwprompt            prompt for a password for the new superuser"
+    echo "  --help                    show this help, then exit"
+    echo "  --version                 output version information, then exit"
+    echo
     echo "Less commonly used options: "
-    echo "  -L DIRECTORY                Where to find the input files"
-    echo "  -d, --debug                 Generate lots of debugging output"
-    echo "  -n, --noclean               Do not clean up after errors"
+    echo "  -d, --debug               generate lots of debugging output"
+    echo "  -L DIRECTORY              where to find the input files"
+    echo "  -n, --noclean             do not clean up after errors"
+    echo
+    echo "If the data directory is not specified, the environment variable PGDATA"
+    echo "is used."
     echo
     echo "Report bugs to <pgsql-bugs@postgresql.org>."
     exit 0
 fi
 
 #-------------------------------------------------------------------------
-# Resolve the multibyte encoding name
+# Resolve the encoding name
 #-------------------------------------------------------------------------
 
-if [ "$MULTIBYTE" ]
+if [ "$ENCODING" ]
 then
-	MULTIBYTEID=`$PGPATH/pg_encoding -b $MULTIBYTE`
-        if [ "$?" -ne 0 ]
+	ENCODINGID=`$PGPATH/pg_encoding -b $ENCODING`
+	if [ "$?" -ne 0 ]
 	then
               (
                 echo "$CMDNAME: pg_encoding failed"
-                echo
-                echo "Perhaps you did not configure PostgreSQL for multibyte support or"
-                echo "the program was not successfully installed."
+                echo "Make sure the program was installed correctly."
               ) 1>&2
                 exit 1
         fi
-	if [ -z "$MULTIBYTEID" ]
+	if [ -z "$ENCODINGID" ]
 	then
-		echo "$CMDNAME: $MULTIBYTE is not a valid backend encoding name" 1>&2
+		echo "$CMDNAME: $ENCODING is not a valid backend encoding name" 1>&2
 		exit 1
 	fi
 fi
@@ -341,7 +418,7 @@ then
   (
     echo
     echo "initdb variables:"
-    for var in PGDATA datadir PGPATH MULTIBYTE MULTIBYTEID \
+    for var in PGDATA datadir PGPATH ENCODING ENCODINGID \
         POSTGRES_SUPERUSERNAME POSTGRES_BKI \
         POSTGRES_DESCR POSTGRESQL_CONF_SAMPLE \
 	PG_HBA_SAMPLE PG_IDENT_SAMPLE ; do
@@ -387,6 +464,30 @@ echo "The files belonging to this database system will be owned by user \"$Effec
 echo "This user must also own the server process."
 echo
 
+TAB='	'
+
+if test x`pg_getlocale CTYPE` = x`pg_getlocale COLLATE` \
+   && test x`pg_getlocale CTYPE` = x`pg_getlocale TIME` \
+   && test x`pg_getlocale CTYPE` = x`pg_getlocale NUMERIC` \
+   && test x`pg_getlocale CTYPE` = x`pg_getlocale MONETARY` \
+   && test x`pg_getlocale CTYPE` = x`pg_getlocale MESSAGES`
+then
+    echo "The database cluster will be initialized with locale `pg_getlocale CTYPE`."
+else
+    echo "The database cluster will be initialized with locales:"
+    echo "    COLLATE:  `pg_getlocale COLLATE`${TAB}CTYPE:   `pg_getlocale CTYPE`${TAB}MESSAGES: `pg_getlocale MESSAGES`"
+    echo "    MONETARY: `pg_getlocale MONETARY`${TAB}NUMERIC: `pg_getlocale NUMERIC`${TAB}TIME:     `pg_getlocale TIME`"
+fi
+
+# (Be sure to maintain the correspondence with locale_is_like_safe() in selfuncs.c.)
+if test x`pg_getlocale COLLATE` != xC && test x`pg_getlocale COLLATE` != xPOSIX; then
+    echo "This locale setting will prevent the use of indexes for pattern matching"
+    echo "operations.  If that is a concern, rerun $CMDNAME with the collation order"
+    echo "set to \"C\".  For more information see the Administrator's Guide."
+fi
+echo
+
+
 ##########################################################################
 #
 # CREATE DATABASE DIRECTORY
@@ -452,7 +553,7 @@ PGSQL_OPT="-F -D$PGDATA"
 
 if [ "$debug" = yes ]
 then
-    BACKEND_TALK_ARG="-d"
+    BACKEND_TALK_ARG="-d 5"
 else
     PGSQL_OPT="$PGSQL_OPT -o /dev/null"
 fi
@@ -468,8 +569,16 @@ echo "$short_version" > "$PGDATA/PG_VERSION" || exit_nicely
 
 cat "$POSTGRES_BKI" \
 | sed -e "s/POSTGRES/$POSTGRES_SUPERUSERNAME/g" \
-      -e "s/ENCODING/$MULTIBYTEID/g" \
-| "$PGPATH"/postgres -boot -x1 $PGSQL_OPT $BACKEND_TALK_ARG template1 \
+      -e "s/ENCODING/$ENCODINGID/g" \
+| 
+(
+  LC_COLLATE=`pg_getlocale COLLATE`
+  LC_CTYPE=`pg_getlocale CTYPE`
+  export LC_COLLATE
+  export LC_CTYPE
+  unset LC_ALL
+  "$PGPATH"/postgres -boot -x1 $PGSQL_OPT $BACKEND_TALK_ARG template1
+) \
 || exit_nicely
 
 # Make the per-database PGVERSION for template1 only after init'ing it
@@ -485,7 +594,19 @@ $ECHO_N "creating configuration files... "$ECHO_C
 
 cp "$PG_HBA_SAMPLE" "$PGDATA"/pg_hba.conf              || exit_nicely
 cp "$PG_IDENT_SAMPLE" "$PGDATA"/pg_ident.conf          || exit_nicely
-cp "$POSTGRESQL_CONF_SAMPLE" "$PGDATA"/postgresql.conf || exit_nicely
+(
+  cat "$POSTGRESQL_CONF_SAMPLE"
+  echo
+  echo
+  echo "#"
+  echo "#	Locale settings"
+  echo "#"
+  echo "# (initialized by initdb -- may be changed)"
+  for cat in MESSAGES MONETARY NUMERIC TIME; do
+    echo "LC_$cat = '`pg_getlocale $cat`'"
+  done
+) > "$PGDATA"/postgresql.conf || exit_nicely
+
 chmod 0600 "$PGDATA"/pg_hba.conf "$PGDATA"/pg_ident.conf \
 	"$PGDATA"/postgresql.conf
 
@@ -500,15 +621,17 @@ echo "ok"
 # To break an SQL command across lines in this script, backslash-escape all
 # internal newlines in the command.
 
-PGSQL_OPT="$PGSQL_OPT -O"
+PGSQL_OPT="$PGSQL_OPT -O -c search_path=pg_catalog"
 
 $ECHO_N "initializing pg_shadow... "$ECHO_C
 
 "$PGPATH"/postgres $PGSQL_OPT template1 >/dev/null <<EOF
 -- Create a trigger so that direct updates to pg_shadow will be written
--- to the flat password file pg_pwd
+-- to the flat password/group files pg_pwd and pg_group
 CREATE TRIGGER pg_sync_pg_pwd AFTER INSERT OR UPDATE OR DELETE ON pg_shadow \
-FOR EACH ROW EXECUTE PROCEDURE update_pg_pwd();
+FOR EACH ROW EXECUTE PROCEDURE update_pg_pwd_and_pg_group();
+CREATE TRIGGER pg_sync_pg_group AFTER INSERT OR UPDATE OR DELETE ON pg_group \
+FOR EACH ROW EXECUTE PROCEDURE update_pg_pwd_and_pg_group();
 -- needs to be done before alter user, because alter user checks that
 -- pg_shadow is secure ...
 REVOKE ALL on pg_shadow FROM public;
@@ -549,15 +672,17 @@ EOF
     echo "ok"
 fi
 
-
 $ECHO_N "enabling unlimited row size for system tables... "$ECHO_C
 
 "$PGPATH"/postgres $PGSQL_OPT template1 >/dev/null <<EOF
 ALTER TABLE pg_attrdef CREATE TOAST TABLE;
+ALTER TABLE pg_constraint CREATE TOAST TABLE;
+ALTER TABLE pg_database CREATE TOAST TABLE;
 ALTER TABLE pg_description CREATE TOAST TABLE;
+ALTER TABLE pg_group CREATE TOAST TABLE;
 ALTER TABLE pg_proc CREATE TOAST TABLE;
-ALTER TABLE pg_relcheck CREATE TOAST TABLE;
 ALTER TABLE pg_rewrite CREATE TOAST TABLE;
+ALTER TABLE pg_shadow CREATE TOAST TABLE;
 ALTER TABLE pg_statistic CREATE TOAST TABLE;
 EOF
 if [ "$?" -ne 0 ]; then
@@ -566,6 +691,38 @@ fi
 echo "ok"
 
 
+$ECHO_N "initializing pg_depend... "$ECHO_C
+
+"$PGPATH"/postgres $PGSQL_OPT template1 >/dev/null <<EOF
+-- Make PIN entries in pg_depend for all objects made so far in the tables
+-- that the dependency code handles.  This is overkill (the system doesn't
+-- really depend on having every last weird datatype, for instance)
+-- but generating only the minimum required set of dependencies seems hard.
+-- Note that we deliberately do not pin the system views.
+-- First delete any already-made entries; PINs override all else, and must
+-- be the only entries for their objects.
+DELETE FROM pg_depend;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_class;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_proc;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_type;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_cast;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_constraint;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_attrdef;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_language;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_operator;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_opclass;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_rewrite;
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_trigger;
+-- restriction here to avoid pinning the public namespace
+INSERT INTO pg_depend SELECT 0,0,0, tableoid,oid,0, 'p' FROM pg_namespace \
+    WHERE nspname LIKE 'pg%';
+EOF
+if [ "$?" -ne 0 ]; then
+    exit_nicely
+fi
+echo "ok"
+
+
 $ECHO_N "creating system views... "$ECHO_C
 
 "$PGPATH"/postgres $PGSQL_OPT template1 >/dev/null <<EOF
@@ -575,54 +732,57 @@ CREATE VIEW pg_user AS \
         usename, \
         usesysid, \
         usecreatedb, \
-        usetrace, \
         usesuper, \
         usecatupd, \
         '********'::text as passwd, \
-        valuntil \
+        valuntil, \
+        useconfig \
     FROM pg_shadow;
 
 CREATE VIEW pg_rules AS \
     SELECT \
+        N.nspname AS schemaname, \
         C.relname AS tablename, \
         R.rulename AS rulename, \
-        pg_get_ruledef(R.rulename) AS definition \
-    FROM pg_rewrite R, pg_class C \
-    WHERE R.rulename !~ '^_RET' \
-        AND C.oid = R.ev_class;
+        pg_get_ruledef(R.oid) AS definition \
+    FROM (pg_rewrite R JOIN pg_class C ON (C.oid = R.ev_class)) \
+        LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
+    WHERE R.rulename != '_RETURN';
 
 CREATE VIEW pg_views AS \
     SELECT \
+        N.nspname AS schemaname, \
         C.relname AS viewname, \
         pg_get_userbyid(C.relowner) AS viewowner, \
-        pg_get_viewdef(C.relname) AS definition \
-    FROM pg_class C \
+        pg_get_viewdef(C.oid) AS definition \
+    FROM pg_class C LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
     WHERE C.relkind = 'v';
 
--- XXX why does pg_tables include sequences?
-
 CREATE VIEW pg_tables AS \
     SELECT \
+        N.nspname AS schemaname, \
         C.relname AS tablename, \
         pg_get_userbyid(C.relowner) AS tableowner, \
         C.relhasindex AS hasindexes, \
         C.relhasrules AS hasrules, \
         (C.reltriggers > 0) AS hastriggers \
-    FROM pg_class C \
+    FROM pg_class C LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
     WHERE C.relkind IN ('r', 's');
 
 CREATE VIEW pg_indexes AS \
     SELECT \
+        N.nspname AS schemaname, \
         C.relname AS tablename, \
         I.relname AS indexname, \
-        pg_get_indexdef(X.indexrelid) AS indexdef \
-    FROM pg_index X, pg_class C, pg_class I \
-    WHERE C.relkind = 'r' AND I.relkind = 'i' \
-        AND C.oid = X.indrelid \
-        AND I.oid = X.indexrelid;
+        pg_get_indexdef(I.oid) AS indexdef \
+    FROM pg_index X JOIN pg_class C ON (C.oid = X.indrelid) \
+         JOIN pg_class I ON (I.oid = X.indexrelid) \
+         LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
+    WHERE C.relkind = 'r' AND I.relkind = 'i';
 
 CREATE VIEW pg_stats AS \
     SELECT \
+        nspname AS schemaname, \
         relname AS tablename, \
         attname AS attname, \
         stanullfrac AS null_frac, \
@@ -652,16 +812,17 @@ CREATE VIEW pg_stats AS \
             WHEN stakind3 THEN stanumbers3[1] \
             WHEN stakind4 THEN stanumbers4[1] \
         END AS correlation \
-    FROM pg_class c, pg_attribute a, pg_statistic s \
-    WHERE c.oid = s.starelid AND c.oid = a.attrelid \
-        AND a.attnum = s.staattnum \
-        AND has_table_privilege(c.oid, 'select');
+    FROM pg_statistic s JOIN pg_class c ON (c.oid = s.starelid) \
+         JOIN pg_attribute a ON (c.oid = attrelid AND attnum = s.staattnum) \
+         LEFT JOIN pg_namespace n ON (n.oid = c.relnamespace) \
+    WHERE has_table_privilege(c.oid, 'select');
 
 REVOKE ALL on pg_statistic FROM public;
 
 CREATE VIEW pg_stat_all_tables AS \
     SELECT \
             C.oid AS relid, \
+            N.nspname AS schemaname, \
             C.relname AS relname, \
             pg_stat_get_numscans(C.oid) AS seq_scan, \
             pg_stat_get_tuples_returned(C.oid) AS seq_tup_read, \
@@ -670,22 +831,24 @@ CREATE VIEW pg_stat_all_tables AS \
             pg_stat_get_tuples_inserted(C.oid) AS n_tup_ins, \
             pg_stat_get_tuples_updated(C.oid) AS n_tup_upd, \
             pg_stat_get_tuples_deleted(C.oid) AS n_tup_del \
-    FROM pg_class C LEFT OUTER JOIN \
+    FROM pg_class C LEFT JOIN \
          pg_index I ON C.oid = I.indrelid \
+         LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
     WHERE C.relkind = 'r' \
-    GROUP BY C.oid, C.relname;
+    GROUP BY C.oid, N.nspname, C.relname;
 
 CREATE VIEW pg_stat_sys_tables AS \
     SELECT * FROM pg_stat_all_tables \
-    WHERE relname ~ '^pg_';
+    WHERE schemaname IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_stat_user_tables AS \
     SELECT * FROM pg_stat_all_tables \
-    WHERE relname !~ '^pg_';
+    WHERE schemaname NOT IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_all_tables AS \
     SELECT \
             C.oid AS relid, \
+            N.nspname AS schemaname, \
             C.relname AS relname, \
             pg_stat_get_blocks_fetched(C.oid) - \
                     pg_stat_get_blocks_hit(C.oid) AS heap_blks_read, \
@@ -699,86 +862,89 @@ CREATE VIEW pg_statio_all_tables AS \
             pg_stat_get_blocks_fetched(X.oid) - \
                     pg_stat_get_blocks_hit(X.oid) AS tidx_blks_read, \
             pg_stat_get_blocks_hit(X.oid) AS tidx_blks_hit \
-    FROM pg_class C LEFT OUTER JOIN \
-            pg_index I ON C.oid = I.indrelid LEFT OUTER JOIN \
-            pg_class T ON C.reltoastrelid = T.oid LEFT OUTER JOIN \
+    FROM pg_class C LEFT JOIN \
+            pg_index I ON C.oid = I.indrelid LEFT JOIN \
+            pg_class T ON C.reltoastrelid = T.oid LEFT JOIN \
             pg_class X ON T.reltoastidxid = X.oid \
+            LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
     WHERE C.relkind = 'r' \
-    GROUP BY C.oid, C.relname, T.oid, X.oid;
+    GROUP BY C.oid, N.nspname, C.relname, T.oid, X.oid;
 
 CREATE VIEW pg_statio_sys_tables AS \
     SELECT * FROM pg_statio_all_tables \
-    WHERE relname ~ '^pg_';
+    WHERE schemaname IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_user_tables AS \
     SELECT * FROM pg_statio_all_tables \
-    WHERE relname !~ '^pg_';
+    WHERE schemaname NOT IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_stat_all_indexes AS \
     SELECT \
             C.oid AS relid, \
             I.oid AS indexrelid, \
+            N.nspname AS schemaname, \
             C.relname AS relname, \
             I.relname AS indexrelname, \
             pg_stat_get_numscans(I.oid) AS idx_scan, \
             pg_stat_get_tuples_returned(I.oid) AS idx_tup_read, \
             pg_stat_get_tuples_fetched(I.oid) AS idx_tup_fetch \
-    FROM pg_class C, \
-            pg_class I, \
-            pg_index X \
-    WHERE C.relkind = 'r' AND \
-            X.indrelid = C.oid AND \
-            X.indexrelid = I.oid;
+    FROM pg_class C JOIN \
+            pg_index X ON C.oid = X.indrelid JOIN \
+            pg_class I ON I.oid = X.indexrelid \
+            LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
+    WHERE C.relkind = 'r';
 
 CREATE VIEW pg_stat_sys_indexes AS \
     SELECT * FROM pg_stat_all_indexes \
-    WHERE relname ~ '^pg_';
+    WHERE schemaname IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_stat_user_indexes AS \
     SELECT * FROM pg_stat_all_indexes \
-    WHERE relname !~ '^pg_';
+    WHERE schemaname NOT IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_all_indexes AS \
     SELECT \
             C.oid AS relid, \
             I.oid AS indexrelid, \
+            N.nspname AS schemaname, \
             C.relname AS relname, \
             I.relname AS indexrelname, \
             pg_stat_get_blocks_fetched(I.oid) - \
                     pg_stat_get_blocks_hit(I.oid) AS idx_blks_read, \
             pg_stat_get_blocks_hit(I.oid) AS idx_blks_hit \
-    FROM pg_class C, \
-            pg_class I, \
-            pg_index X \
-    WHERE C.relkind = 'r' AND \
-            X.indrelid = C.oid AND \
-            X.indexrelid = I.oid;
+    FROM pg_class C JOIN \
+            pg_index X ON C.oid = X.indrelid JOIN \
+            pg_class I ON I.oid = X.indexrelid \
+            LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
+    WHERE C.relkind = 'r';
 
 CREATE VIEW pg_statio_sys_indexes AS \
     SELECT * FROM pg_statio_all_indexes \
-    WHERE relname ~ '^pg_';
+    WHERE schemaname IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_user_indexes AS \
     SELECT * FROM pg_statio_all_indexes \
-    WHERE relname !~ '^pg_';
+    WHERE schemaname NOT IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_all_sequences AS \
     SELECT \
             C.oid AS relid, \
+            N.nspname AS schemaname, \
             C.relname AS relname, \
             pg_stat_get_blocks_fetched(C.oid) - \
                     pg_stat_get_blocks_hit(C.oid) AS blks_read, \
             pg_stat_get_blocks_hit(C.oid) AS blks_hit \
     FROM pg_class C \
+            LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) \
     WHERE C.relkind = 'S';
 
 CREATE VIEW pg_statio_sys_sequences AS \
     SELECT * FROM pg_statio_all_sequences \
-    WHERE relname ~ '^pg_';
+    WHERE schemaname IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_statio_user_sequences AS \
     SELECT * FROM pg_statio_all_sequences \
-    WHERE relname !~ '^pg_';
+    WHERE schemaname NOT IN ('pg_catalog', 'pg_toast');
 
 CREATE VIEW pg_stat_activity AS \
     SELECT \
@@ -806,6 +972,24 @@ CREATE VIEW pg_stat_database AS \
             pg_stat_get_db_blocks_hit(D.oid) AS blks_hit \
     FROM pg_database D;
 
+CREATE VIEW pg_locks AS \
+    SELECT * \
+    FROM pg_lock_status() AS L(relation oid, database oid, \
+	transaction xid, pid int4, mode text, granted boolean);
+
+CREATE VIEW pg_settings AS \
+    SELECT * \
+    FROM pg_show_all_settings() AS A(name text, setting text);
+
+CREATE RULE pg_settings_u AS \
+    ON UPDATE TO pg_settings \
+    WHERE new.name = old.name DO \
+    SELECT set_config(old.name, new.setting, 'f');
+
+CREATE RULE pg_settings_n AS \
+    ON UPDATE TO pg_settings \
+    DO INSTEAD NOTHING;
+
 EOF
 if [ "$?" -ne 0 ]; then
     exit_nicely
@@ -833,9 +1017,35 @@ EOF
 	| "$PGPATH"/postgres $PGSQL_OPT template1 > /dev/null || exit_nicely
 echo "ok"
 
+# Create pg_conversion and support functions
+$ECHO_N "creating conversions... "$ECHO_C
+grep -v '^DROP CONVERSION' $datadir/conversion_create.sql | "$PGPATH"/postgres $PGSQL_OPT template1 > /dev/null || exit_nicely
+echo "ok"
+
+# Set most system catalogs and built-in functions as world-accessible.
+# Some objects may require different permissions by default, so we
+# make sure we don't overwrite privilege sets that have already been
+# set (NOT NULL).
+$ECHO_N "setting privileges on built-in objects... "$ECHO_C
+(
+  cat <<EOF
+    UPDATE pg_class SET relacl = '{"=r"}' \
+        WHERE relkind IN ('r', 'v', 'S') AND relacl IS NULL;
+    UPDATE pg_proc SET proacl = '{"=X"}' \
+        WHERE proacl IS NULL;
+    UPDATE pg_language SET lanacl = '{"=U"}' \
+        WHERE lanpltrusted;
+    UPDATE pg_language SET lanacl = '{"="}' \
+        WHERE NOT lanpltrusted;
+EOF
+) \
+	| "$PGPATH"/postgres $PGSQL_OPT template1 > /dev/null || exit_nicely
+echo "ok"
+
 $ECHO_N "vacuuming database template1... "$ECHO_C
 
 "$PGPATH"/postgres $PGSQL_OPT template1 >/dev/null <<EOF
+ANALYZE;
 VACUUM FULL FREEZE;
 EOF
 if [ "$?" -ne 0 ]; then
@@ -856,7 +1066,15 @@ UPDATE pg_database SET \
 -- We use the OID of template0 to determine lastsysoid
 
 UPDATE pg_database SET datlastsysoid = \
-    (SELECT oid - 1 FROM pg_database WHERE datname = 'template0');
+    (SELECT oid::int4 - 1 FROM pg_database WHERE datname = 'template0');
+
+-- Explicitly revoke public create-schema and create-temp-table privileges
+-- in template1 and template0; else the latter would be on by default
+
+REVOKE CREATE,TEMPORARY ON DATABASE template1 FROM public;
+REVOKE CREATE,TEMPORARY ON DATABASE template0 FROM public;
+
+-- Finally vacuum to clean up dead rows in pg_database
 
 VACUUM FULL pg_database;
 EOF
