@@ -5,25 +5,7 @@
  *	Implements the basic DB functions used by the archiver.
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_db.c,v 1.31 2002/01/18 19:17:05 momjian Exp $
- *
- * NOTES
- *
- * Modifications - 04-Jan-2001 - pjw@rhyme.com.au
- *
- *	  - Check results of PQ routines more carefully.
- *
- * Modifications - 19-Mar-2001 - pjw@rhyme.com.au
- *
- *	  - Avoid forcing table name to lower case in FixupBlobXrefs!
- *
- *
- * Modifications - 18-Jan-2002 - pjw@rhyme.com.au
- *
- *	  - Split ExecuteSqlCommandBuf into 3 routines for (slightly) improved
- *		clarity. Modify loop to cater for COPY commands buried in the SQL
- *		command buffer (prev version assumed COPY command was executed
- *		in prior call). This was to fix the buf in the 'set max oid' code.
+ *	  $Header: /cvsroot/pgsql-server/src/bin/pg_dump/pg_backup_db.c,v 1.43 2002/10/22 19:15:23 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -31,8 +13,9 @@
 #include "pg_backup.h"
 #include "pg_backup_archiver.h"
 #include "pg_backup_db.h"
+#include "dumputils.h"
 
-#include <unistd.h>				/* for getopt() */
+#include <unistd.h>
 #include <ctype.h>
 
 #ifdef HAVE_TERMIOS_H
@@ -55,131 +38,16 @@ static char *_sendSQLLine(ArchiveHandle *AH, char *qry, char *eos);
 static char *_sendCopyLine(ArchiveHandle *AH, char *qry, char *eos);
 
 
-/*
- * simple_prompt  --- borrowed from psql
- *
- * Generalized function especially intended for reading in usernames and
- * password interactively. Reads from /dev/tty or stdin/stderr.
- *
- * prompt:		The prompt to print
- * maxlen:		How many characters to accept
- * echo:		Set to false if you want to hide what is entered (for passwords)
- *
- * Returns a malloc()'ed string with the input (w/o trailing newline).
- */
-static bool prompt_state = false;
-
-char *
-simple_prompt(const char *prompt, int maxlen, bool echo)
-{
-	int			length;
-	char	   *destination;
-	FILE	   *termin,
-			   *termout;
-
-#ifdef HAVE_TERMIOS_H
-	struct termios t_orig,
-				t;
-#endif
-
-	destination = (char *) malloc(maxlen + 2);
-	if (!destination)
-		return NULL;
-
-	prompt_state = true;		/* disable SIGINT */
-
-	/*
-	 * Do not try to collapse these into one "w+" mode file. Doesn't work
-	 * on some platforms (eg, HPUX 10.20).
-	 */
-	termin = fopen("/dev/tty", "r");
-	termout = fopen("/dev/tty", "w");
-	if (!termin || !termout)
-	{
-		if (termin)
-			fclose(termin);
-		if (termout)
-			fclose(termout);
-		termin = stdin;
-		termout = stderr;
-	}
-
-#ifdef HAVE_TERMIOS_H
-	if (!echo)
-	{
-		tcgetattr(fileno(termin), &t);
-		t_orig = t;
-		t.c_lflag &= ~ECHO;
-		tcsetattr(fileno(termin), TCSAFLUSH, &t);
-	}
-#endif
-
-	if (prompt)
-	{
-		fputs(gettext(prompt), termout);
-		fflush(termout);
-	}
-
-	if (fgets(destination, maxlen, termin) == NULL)
-		destination[0] = '\0';
-
-	length = strlen(destination);
-	if (length > 0 && destination[length - 1] != '\n')
-	{
-		/* eat rest of the line */
-		char		buf[128];
-		int			buflen;
-
-		do
-		{
-			if (fgets(buf, sizeof(buf), termin) == NULL)
-				break;
-			buflen = strlen(buf);
-		} while (buflen > 0 && buf[buflen - 1] != '\n');
-	}
-
-	if (length > 0 && destination[length - 1] == '\n')
-		/* remove trailing newline */
-		destination[length - 1] = '\0';
-
-#ifdef HAVE_TERMIOS_H
-	if (!echo)
-	{
-		tcsetattr(fileno(termin), TCSAFLUSH, &t_orig);
-		fputs("\n", termout);
-		fflush(termout);
-	}
-#endif
-
-	if (termin != stdin)
-	{
-		fclose(termin);
-		fclose(termout);
-	}
-
-	prompt_state = false;		/* SIGINT okay again */
-
-	return destination;
-}
-
-
 static int
 _parse_version(ArchiveHandle *AH, const char *versionString)
 {
-	int			cnt;
-	int			vmaj,
-				vmin,
-				vrev;
+	int			v;
 
-	cnt = sscanf(versionString, "%d.%d.%d", &vmaj, &vmin, &vrev);
-
-	if (cnt < 2)
+	v = parse_version(versionString);
+	if (v < 0)
 		die_horribly(AH, modulename, "unable to parse version string \"%s\"\n", versionString);
 
-	if (cnt == 2)
-		vrev = 0;
-
-	return (100 * vmaj + vmin) * 100 + vrev;
+	return v;
 }
 
 static void
@@ -193,11 +61,15 @@ _check_database_version(ArchiveHandle *AH, bool ignoreVersion)
 
 	myversion = _parse_version(AH, PG_VERSION);
 
-	res = PQexec(conn, "SELECT version();");
+	/*
+	 *	Autocommit could be off.  We turn it off later but we have to check
+	 *	the database version first.
+	 */
+
+	res = PQexec(conn, "BEGIN;SELECT version();");
 	if (!res ||
 		PQresultStatus(res) != PGRES_TUPLES_OK ||
 		PQntuples(res) != 1)
-
 		die_horribly(AH, modulename, "could not get version from server: %s", PQerrorMessage(conn));
 
 	remoteversion_str = PQgetvalue(res, 0, 0);
@@ -205,6 +77,12 @@ _check_database_version(ArchiveHandle *AH, bool ignoreVersion)
 
 	PQclear(res);
 
+	res = PQexec(conn, "COMMIT;");
+	if (!res ||
+		PQresultStatus(res) != PGRES_COMMAND_OK)
+		die_horribly(AH, modulename, "could not get version from server: %s", PQerrorMessage(conn));
+	PQclear(res);
+
 	AH->public.remoteVersion = remoteversion;
 
 	if (myversion != remoteversion
@@ -219,57 +97,6 @@ _check_database_version(ArchiveHandle *AH, bool ignoreVersion)
 	}
 }
 
-/*
- * Check if a given user is a superuser.
- */
-int
-UserIsSuperuser(ArchiveHandle *AH, char *user)
-{
-	PQExpBuffer qry = createPQExpBuffer();
-	PGresult   *res;
-	int			i_usesuper;
-	int			ntups;
-	int			isSuper;
-
-	/* Get the superuser setting */
-	appendPQExpBuffer(qry, "select usesuper from pg_user where usename = '%s'", user);
-	res = PQexec(AH->connection, qry->data);
-
-	if (!res)
-		die_horribly(AH, modulename, "null result checking superuser status of %s\n", user);
-
-	if (PQresultStatus(res) != PGRES_TUPLES_OK)
-		die_horribly(AH, modulename, "could not check superuser status of %s: %s",
-					 user, PQerrorMessage(AH->connection));
-
-	ntups = PQntuples(res);
-
-	if (ntups == 0)
-		isSuper = 0;
-	else
-	{
-		i_usesuper = PQfnumber(res, "usesuper");
-		isSuper = (strcmp(PQgetvalue(res, 0, i_usesuper), "t") == 0);
-	}
-	PQclear(res);
-
-	destroyPQExpBuffer(qry);
-
-	return isSuper;
-}
-
-int
-ConnectedUserIsSuperuser(ArchiveHandle *AH)
-{
-	return UserIsSuperuser(AH, PQuser(AH->connection));
-}
-
-char *
-ConnectedUser(ArchiveHandle *AH)
-{
-	return PQuser(AH->connection);
-}
-
 /*
  * Reconnect to the server.  If dbname is not NULL, use that database,
  * else the one associated with the archive handle.  If username is
@@ -306,9 +133,10 @@ ReconnectToServer(ArchiveHandle *AH, const char *dbname, const char *username)
 	PQfinish(AH->connection);
 	AH->connection = newConn;
 
-	free(AH->username);
-	AH->username = strdup(newusername);
-	/* XXX Why don't we update AH->dbname? */
+	/* don't assume we still know the output schema */
+	if (AH->currSchema)
+		free(AH->currSchema);
+	AH->currSchema = strdup("");
 
 	return 1;
 }
@@ -380,7 +208,6 @@ _connectDB(ArchiveHandle *AH, const char *reqdb, const char *requser)
 				die_horribly(AH, modulename, "could not reconnect to database: %s",
 							 PQerrorMessage(newConn));
 		}
-
 	} while (need_pass);
 
 	if (password)
@@ -413,26 +240,6 @@ ConnectDatabase(Archive *AHX,
 	if (AH->connection)
 		die_horribly(AH, modulename, "already connected to a database\n");
 
-	if (!dbname && !(dbname = getenv("PGDATABASE")))
-		die_horribly(AH, modulename, "no database name specified\n");
-
-	AH->dbname = strdup(dbname);
-
-	if (pghost != NULL)
-		AH->pghost = strdup(pghost);
-	else
-		AH->pghost = NULL;
-
-	if (pgport != NULL)
-		AH->pgport = strdup(pgport);
-	else
-		AH->pgport = NULL;
-
-	if (username != NULL)
-		AH->username = strdup(username);
-	else
-		AH->username = NULL;
-
 	if (reqPwd)
 	{
 		password = simple_prompt("Password: ", 100, false);
@@ -450,8 +257,8 @@ ConnectDatabase(Archive *AHX,
 	do
 	{
 		need_pass = false;
-		AH->connection = PQsetdbLogin(AH->pghost, AH->pgport, NULL, NULL,
-									  AH->dbname, AH->username, password);
+		AH->connection = PQsetdbLogin(pghost, pgport, NULL, NULL,
+									  dbname, username, password);
 
 		if (!AH->connection)
 			die_horribly(AH, modulename, "failed to connect to database\n");
@@ -474,19 +281,24 @@ ConnectDatabase(Archive *AHX,
 	/* check to see that the backend connection was successfully made */
 	if (PQstatus(AH->connection) == CONNECTION_BAD)
 		die_horribly(AH, modulename, "connection to database \"%s\" failed: %s",
-					 AH->dbname, PQerrorMessage(AH->connection));
+				   PQdb(AH->connection), PQerrorMessage(AH->connection));
 
 	/* check for version mismatch */
 	_check_database_version(AH, ignoreVersion);
 
-	PQsetNoticeProcessor(AH->connection, notice_processor, NULL);
+	/* Turn autocommit on */
+	if (AH->public.remoteVersion >= 70300)
+	{
+		PGresult   *res;
 
-	/*
-	 * AH->currUser = PQuser(AH->connection);
-	 *
-	 * Removed because it prevented an initial \connect when dumping to SQL
-	 * in pg_dump.
-	 */
+		res = PQexec(AH->connection, "SET autocommit TO 'on'");
+		if (!res || PQresultStatus(res) != PGRES_COMMAND_OK)
+			die_horribly(AH, NULL, "SET autocommit TO 'on' failed: %s",
+						  PQerrorMessage(AH->connection));
+		PQclear(res);
+	}
+
+	PQsetNoticeProcessor(AH->connection, notice_processor, NULL);
 
 	return AH->connection;
 }
@@ -550,7 +362,7 @@ _executeSqlCommand(ArchiveHandle *AH, PGconn *conn, PQExpBuffer qry, char *desc)
 static char *
 _sendCopyLine(ArchiveHandle *AH, char *qry, char *eos)
 {
-	int			loc;			/* Location of next newline */
+	size_t		loc;			/* Location of next newline */
 	int			pos = 0;		/* Current position */
 	int			sPos = 0;		/* Last pos of a slash char */
 	int			isEnd = 0;
@@ -656,7 +468,6 @@ _sendSQLLine(ArchiveHandle *AH, char *qry, char *eos)
 		{
 
 			case SQL_SCAN:		/* Default state == 0, set in _allocAH */
-
 				if (qry[pos] == ';' && AH->sqlparse.braceDepth == 0)
 				{
 					/* Send It & reset the buffer */
@@ -699,23 +510,19 @@ _sendSQLLine(ArchiveHandle *AH, char *qry, char *eos)
 
 					AH->sqlparse.lastChar = qry[pos];
 				}
-
 				break;
 
 			case SQL_IN_SQL_COMMENT:
-
 				if (qry[pos] == '\n')
 					AH->sqlparse.state = SQL_SCAN;
 				break;
 
 			case SQL_IN_EXT_COMMENT:
-
 				if (AH->sqlparse.lastChar == '*' && qry[pos] == '/')
 					AH->sqlparse.state = SQL_SCAN;
 				break;
 
 			case SQL_IN_QUOTE:
-
 				if (!AH->sqlparse.backSlash && AH->sqlparse.quoteChar == qry[pos])
 				{
 					/* fprintf(stderr,"[endquote]\n"); */
@@ -746,13 +553,12 @@ _sendSQLLine(ArchiveHandle *AH, char *qry, char *eos)
 	 * stmt
 	 */
 	return eos;
-
 }
 
 
 /* Convenience function to send one or more queries. Monitors result to handle COPY statements */
 int
-ExecuteSqlCommandBuf(ArchiveHandle *AH, void *qryv, int bufLen)
+ExecuteSqlCommandBuf(ArchiveHandle *AH, void *qryv, size_t bufLen)
 {
 	char	   *qry = (char *) qryv;
 	char	   *eos = qry + bufLen;
@@ -775,8 +581,9 @@ ExecuteSqlCommandBuf(ArchiveHandle *AH, void *qryv, int bufLen)
 }
 
 void
-FixupBlobRefs(ArchiveHandle *AH, char *tablename)
+FixupBlobRefs(ArchiveHandle *AH, TocEntry *te)
 {
+	PQExpBuffer tblName;
 	PQExpBuffer tblQry;
 	PGresult   *res,
 			   *uRes;
@@ -784,44 +591,55 @@ FixupBlobRefs(ArchiveHandle *AH, char *tablename)
 				n;
 	char	   *attr;
 
-	if (strcmp(tablename, BLOB_XREF_TABLE) == 0)
+	if (strcmp(te->tag, BLOB_XREF_TABLE) == 0)
 		return;
 
+	tblName = createPQExpBuffer();
 	tblQry = createPQExpBuffer();
 
-	appendPQExpBuffer(tblQry, "SELECT a.attname FROM pg_class c, pg_attribute a, pg_type t "
-	 " WHERE a.attnum > 0 AND a.attrelid = c.oid AND a.atttypid = t.oid "
-	 " AND t.typname in ('oid', 'lo') AND c.relname = '%s';", tablename);
+	if (te->namespace && strlen(te->namespace) > 0)
+		appendPQExpBuffer(tblName, "%s.",
+						  fmtId(te->namespace));
+	appendPQExpBuffer(tblName, "%s",
+					  fmtId(te->tag));
+
+	appendPQExpBuffer(tblQry,
+					  "SELECT a.attname FROM "
+					  "pg_catalog.pg_attribute a, pg_catalog.pg_type t "
+		 "WHERE a.attnum > 0 AND a.attrelid = '%s'::pg_catalog.regclass "
+				 "AND a.atttypid = t.oid AND t.typname in ('oid', 'lo')",
+					  tblName->data);
 
 	res = PQexec(AH->blobConnection, tblQry->data);
 	if (!res)
 		die_horribly(AH, modulename, "could not find oid columns of table \"%s\": %s",
-					 tablename, PQerrorMessage(AH->connection));
+					 te->tag, PQerrorMessage(AH->connection));
 
 	if ((n = PQntuples(res)) == 0)
 	{
 		/* nothing to do */
-		ahlog(AH, 1, "no OID type columns in table %s\n", tablename);
+		ahlog(AH, 1, "no OID type columns in table %s\n", te->tag);
 	}
 
 	for (i = 0; i < n; i++)
 	{
 		attr = PQgetvalue(res, i, 0);
 
-		ahlog(AH, 1, "fixing large object cross-references for %s.%s\n", tablename, attr);
+		ahlog(AH, 1, "fixing large object cross-references for %s.%s\n",
+			  te->tag, attr);
 
 		resetPQExpBuffer(tblQry);
 
-		/*
-		 * We should use coalesce here (rather than 'exists'), but it
-		 * seems to be broken in 7.0.2 (weird optimizer strategy)
-		 */
-		appendPQExpBuffer(tblQry, "UPDATE \"%s\" SET \"%s\" = ", tablename, attr);
-		appendPQExpBuffer(tblQry, " (SELECT x.newOid FROM \"%s\" x WHERE x.oldOid = \"%s\".\"%s\")",
-						  BLOB_XREF_TABLE, tablename, attr);
-		appendPQExpBuffer(tblQry, " where exists"
-				  "(select * from %s x where x.oldOid = \"%s\".\"%s\");",
-						  BLOB_XREF_TABLE, tablename, attr);
+		/* Can't use fmtId twice in one call... */
+		appendPQExpBuffer(tblQry,
+						  "UPDATE %s SET %s = %s.newOid",
+						  tblName->data, fmtId(attr),
+						  BLOB_XREF_TABLE);
+		appendPQExpBuffer(tblQry,
+						  " FROM %s WHERE %s.oldOid = %s.%s",
+						  BLOB_XREF_TABLE,
+						  BLOB_XREF_TABLE,
+						  tblName->data, fmtId(attr));
 
 		ahlog(AH, 10, "SQL: %s\n", tblQry->data);
 
@@ -829,17 +647,18 @@ FixupBlobRefs(ArchiveHandle *AH, char *tablename)
 		if (!uRes)
 			die_horribly(AH, modulename,
 					"could not update column \"%s\" of table \"%s\": %s",
-					attr, tablename, PQerrorMessage(AH->blobConnection));
+					  attr, te->tag, PQerrorMessage(AH->blobConnection));
 
 		if (PQresultStatus(uRes) != PGRES_COMMAND_OK)
 			die_horribly(AH, modulename,
 				"error while updating column \"%s\" of table \"%s\": %s",
-					attr, tablename, PQerrorMessage(AH->blobConnection));
+					  attr, te->tag, PQerrorMessage(AH->blobConnection));
 
 		PQclear(uRes);
 	}
 
 	PQclear(res);
+	destroyPQExpBuffer(tblName);
 	destroyPQExpBuffer(tblQry);
 }
 
@@ -857,7 +676,7 @@ CreateBlobXrefTable(ArchiveHandle *AH)
 
 	ahlog(AH, 1, "creating table for large object cross-references\n");
 
-	appendPQExpBuffer(qry, "Create Temporary Table %s(oldOid oid, newOid oid);", BLOB_XREF_TABLE);
+	appendPQExpBuffer(qry, "Create Temporary Table %s(oldOid pg_catalog.oid, newOid pg_catalog.oid);", BLOB_XREF_TABLE);
 
 	ExecuteSqlCommand(AH, qry, "could not create large object cross-reference table", true);
 
@@ -870,11 +689,13 @@ CreateBlobXrefTable(ArchiveHandle *AH)
 }
 
 void
-InsertBlobXref(ArchiveHandle *AH, int old, int new)
+InsertBlobXref(ArchiveHandle *AH, Oid old, Oid new)
 {
 	PQExpBuffer qry = createPQExpBuffer();
 
-	appendPQExpBuffer(qry, "Insert Into %s(oldOid, newOid) Values (%d, %d);", BLOB_XREF_TABLE, old, new);
+	appendPQExpBuffer(qry,
+				   "Insert Into %s(oldOid, newOid) Values ('%u', '%u');",
+					  BLOB_XREF_TABLE, old, new);
 
 	ExecuteSqlCommand(AH, qry, "could not create large object cross-reference entry", true);
 
